// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Exponential Moving Average. The Exponential Moving Average is an indicator that shows the average value of a security's price over a period of time. When calculating a moving average. The EMA applies more weight to recent prices than the SMA.
	/// </summary>
	[Description("The Exponential Moving Average is an indicator that shows the average value of a security's price over a period of time. When calculating a moving average. The EMA applies more weight to recent prices than the SMA.")]
	[Gui.Design.DisplayName("TheForceEMA (Exponential Moving Average)")]
	public class TheForceEMA : Indicator
	{
		#region Variables
		private int			period		= 5;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.Cyan, 2), PlotStyle.Line, "EmaClose"));
			Add(new Plot(new Pen(Color.Magenta, 2), PlotStyle.Line, "EmaOpen"));

			Overlay				= true;
			PriceTypeSupported	= false;
			
			DisplayInDataBox = false; 
            CalculateOnBarClose = false;
			PaintPriceMarkers = false; 
		}
		
		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			EmaClose.Set(CurrentBar == 0 ? Close[0] : Close[0] * (2.0 / (1 + Period)) + (1 - (2.0 / (1 + Period))) * EmaClose[1]);
			EmaOpen.Set(CurrentBar == 0 ? Open[0] : Open[0] * (2.0 / (1 + Period)) + (1 - (2.0 / (1 + Period))) * EmaOpen[1]);
		}

		#region Properties
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries EmaClose
		{
			get { return Values[0]; }
		}

		/// <summary>
		/// Gets the slow K value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries EmaOpen
		{
			get { return Values[1]; }
		}
		
		[Description("Numbers of bars used for calculations")]
		[Category("Parameters")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(1, value); }
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TheForceEMA[] cacheTheForceEMA = null;

        private static TheForceEMA checkTheForceEMA = new TheForceEMA();

        /// <summary>
        /// The Exponential Moving Average is an indicator that shows the average value of a security's price over a period of time. When calculating a moving average. The EMA applies more weight to recent prices than the SMA.
        /// </summary>
        /// <returns></returns>
        public TheForceEMA TheForceEMA(int period)
        {
            return TheForceEMA(Input, period);
        }

        /// <summary>
        /// The Exponential Moving Average is an indicator that shows the average value of a security's price over a period of time. When calculating a moving average. The EMA applies more weight to recent prices than the SMA.
        /// </summary>
        /// <returns></returns>
        public TheForceEMA TheForceEMA(Data.IDataSeries input, int period)
        {
            checkTheForceEMA.Period = period;
            period = checkTheForceEMA.Period;

            if (cacheTheForceEMA != null)
                for (int idx = 0; idx < cacheTheForceEMA.Length; idx++)
                    if (cacheTheForceEMA[idx].Period == period && cacheTheForceEMA[idx].EqualsInput(input))
                        return cacheTheForceEMA[idx];

            TheForceEMA indicator = new TheForceEMA();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Period = period;
            indicator.SetUp();

            TheForceEMA[] tmp = new TheForceEMA[cacheTheForceEMA == null ? 1 : cacheTheForceEMA.Length + 1];
            if (cacheTheForceEMA != null)
                cacheTheForceEMA.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTheForceEMA = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The Exponential Moving Average is an indicator that shows the average value of a security's price over a period of time. When calculating a moving average. The EMA applies more weight to recent prices than the SMA.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TheForceEMA TheForceEMA(int period)
        {
            return _indicator.TheForceEMA(Input, period);
        }

        /// <summary>
        /// The Exponential Moving Average is an indicator that shows the average value of a security's price over a period of time. When calculating a moving average. The EMA applies more weight to recent prices than the SMA.
        /// </summary>
        /// <returns></returns>
        public Indicator.TheForceEMA TheForceEMA(Data.IDataSeries input, int period)
        {
            return _indicator.TheForceEMA(input, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The Exponential Moving Average is an indicator that shows the average value of a security's price over a period of time. When calculating a moving average. The EMA applies more weight to recent prices than the SMA.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TheForceEMA TheForceEMA(int period)
        {
            return _indicator.TheForceEMA(Input, period);
        }

        /// <summary>
        /// The Exponential Moving Average is an indicator that shows the average value of a security's price over a period of time. When calculating a moving average. The EMA applies more weight to recent prices than the SMA.
        /// </summary>
        /// <returns></returns>
        public Indicator.TheForceEMA TheForceEMA(Data.IDataSeries input, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TheForceEMA(input, period);
        }

    }
}
#endregion
