// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// The MACD (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.
	/// </summary>
	[Description("The TheForceMACD (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.")]
	public class TheForceMACD : Indicator
	{
		#region Variables
		private int				fast	= 12;
		private int				slow	= 26;
		private int				smooth	= 1;
		private	DataSeries		macd;
		private	DataSeries		fastEma;
		private	DataSeries		slowEma;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.Lime, 4), PlotStyle.Bar, "MacdUp"));
			Add(new Plot(new Pen(Color.Red, 4), PlotStyle.Bar, "MacdDown"));
			Add(new Plot(new Pen(Color.Yellow, 4), PlotStyle.Bar, "MacdDown"));
			Add(new Plot(new Pen(Color.DarkGray, 1), PlotStyle.Line, "Avg"));

			Add(new Line(Color.DarkGray, 0, "Zero line"));

			macd	= new DataSeries(this);
			fastEma	= new DataSeries(this);
			slowEma	= new DataSeries(this);
			
			DisplayInDataBox = false; 
            CalculateOnBarClose = false;
			PaintPriceMarkers = false; 
		}

		/// <summary>
		/// Calculates the indicator value(s) at the current index.
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (CurrentBar == 0)
			{
				fastEma.Set(Input[0]);
				slowEma.Set(Input[0]);
				macd.Set(0);
				MacdNeutral.Set(0);
				Avg.Set(0);
			}
			else
			{
				fastEma.Set((2.0 / (1 + Fast)) * Input[0] + (1 - (2.0 / (1 + Fast))) * fastEma[1]);
				slowEma.Set((2.0 / (1 + Slow)) * Input[0] + (1 - (2.0 / (1 + Slow))) * slowEma[1]);

				double macdtemp		= fastEma[0] - slowEma[0];
				double macdAvg	= (2.0 / (1 + Smooth)) * macdtemp + (1 - (2.0 / (1 + Smooth))) * Avg[1];
				
				macd.Set(macdtemp);
				if (macd[0] > macd[1]) MacdUp.Set(macd[0]);
				else if (macd[0] < macd[1]) MacdDown.Set(macd[0]);
				else if (macd[0] == macd[1]) MacdNeutral.Set(macd[0]);
				
				Avg.Set(macdAvg);
			}
		}

		#region Properties
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Macd
		{
			get { return macd; }
		}
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries MacdUp
		{
			get { return Values[0]; }
		}
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries MacdDown
		{
			get { return Values[1]; }
		}
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries MacdNeutral
		{
			get { return Values[2]; }
		}
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Avg
		{
			get { return Values[3]; }
		}
		
		[Description("Number of bars for fast EMA")]
		[Category("Parameters")]
		public int Fast
		{
			get { return fast; }
			set { fast = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for slow EMA")]
		[Category("Parameters")]
		public int Slow
		{
			get { return slow; }
			set { slow = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for smoothing")]
		[Category("Parameters")]
		public int Smooth
		{
			get { return smooth; }
			set { smooth = Math.Max(1, value); }
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TheForceMACD[] cacheTheForceMACD = null;

        private static TheForceMACD checkTheForceMACD = new TheForceMACD();

        /// <summary>
        /// The TheForceMACD (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        public TheForceMACD TheForceMACD(int fast, int slow, int smooth)
        {
            return TheForceMACD(Input, fast, slow, smooth);
        }

        /// <summary>
        /// The TheForceMACD (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        public TheForceMACD TheForceMACD(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            checkTheForceMACD.Fast = fast;
            fast = checkTheForceMACD.Fast;
            checkTheForceMACD.Slow = slow;
            slow = checkTheForceMACD.Slow;
            checkTheForceMACD.Smooth = smooth;
            smooth = checkTheForceMACD.Smooth;

            if (cacheTheForceMACD != null)
                for (int idx = 0; idx < cacheTheForceMACD.Length; idx++)
                    if (cacheTheForceMACD[idx].Fast == fast && cacheTheForceMACD[idx].Slow == slow && cacheTheForceMACD[idx].Smooth == smooth && cacheTheForceMACD[idx].EqualsInput(input))
                        return cacheTheForceMACD[idx];

            TheForceMACD indicator = new TheForceMACD();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Fast = fast;
            indicator.Slow = slow;
            indicator.Smooth = smooth;
            indicator.SetUp();

            TheForceMACD[] tmp = new TheForceMACD[cacheTheForceMACD == null ? 1 : cacheTheForceMACD.Length + 1];
            if (cacheTheForceMACD != null)
                cacheTheForceMACD.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTheForceMACD = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The TheForceMACD (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TheForceMACD TheForceMACD(int fast, int slow, int smooth)
        {
            return _indicator.TheForceMACD(Input, fast, slow, smooth);
        }

        /// <summary>
        /// The TheForceMACD (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        public Indicator.TheForceMACD TheForceMACD(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            return _indicator.TheForceMACD(input, fast, slow, smooth);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The TheForceMACD (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TheForceMACD TheForceMACD(int fast, int slow, int smooth)
        {
            return _indicator.TheForceMACD(Input, fast, slow, smooth);
        }

        /// <summary>
        /// The TheForceMACD (Moving Average Convergence/Divergence) is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        public Indicator.TheForceMACD TheForceMACD(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TheForceMACD(input, fast, slow, smooth);
        }

    }
}
#endregion
