// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// The Stochastic Oscillator is made up of two lines that oscillate between a vertical scale of 0 to 100. The %K is the main line and it is drawn as a solid line. The second is the %D line and is a moving average of %K. The %D line is drawn as a dotted line. Use as a buy/sell signal generator, buying when fast moves above slow and selling when fast moves below slow.
	/// </summary>
	[Description("The TheForceStochastics Oscillator is made up of two lines that oscillate between a vertical scale of 0 to 100. The %K is the main line and it is drawn as a solid line. The second is the %D line and is a moving average of %K. The %D line is drawn as a dotted line. Use as a buy/sell signal generator, buying when fast moves above slow and selling when fast moves below slow.")]
	public class TheForceStochastics : Indicator
	{
		#region Variables
		private int				periodD	= 3;	// SlowDperiod
		private int				periodK	= 5;	// Kperiod
		private int				smooth	= 3;	// SlowKperiod
		private DataSeries		den;
		private DataSeries		nom;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.Magenta, 2), PlotStyle.Line, "D"));
			Add(new Plot(new Pen(Color.Cyan, 2), PlotStyle.Line, "K"));
			Plots[0].Pen.DashStyle = DashStyle.Dash;

			Add(new Line(Color.DarkGray, 20, "Lower"));
			Add(new Line(Color.DarkGray, 80, "Upper"));

			den			= new DataSeries(this);
			nom			= new DataSeries(this);

			DisplayInDataBox = false; 
            CalculateOnBarClose = false;
			PaintPriceMarkers = false; 
			}

		/// <summary>
		/// Calculates the indicator value(s) at the current index.
		/// </summary>
		protected override void OnBarUpdate()
		{
			nom.Set(Close[0] - MIN(Low, PeriodK)[0]);
			den.Set(MAX(High, PeriodK)[0] - MIN(Low, PeriodK)[0]);										// PeriodK = Kperiod
			
			K.Set(100 * SUM(nom, Smooth)[0] / (SUM(den, Smooth)[0] == 0 ? 1.0 : SUM(den, Smooth)[0]));	// Smooth  = SlowKperiod
			D.Set(SMA(K, PeriodD)[0]);																	// PeriodD = SlowDperiod
		}

		#region Properties
		/// <summary>
		/// Gets the slow D value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries D
		{
			get { return Values[0]; }
		}

		/// <summary>
		/// Gets the slow K value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries K
		{
			get { return Values[1]; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for moving average over K values")]
		[Category("Parameters")]
		public int PeriodD
		{
			get { return periodD; }
			set { periodD = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for calculating the K values")]
		[Category("Parameters")]
		public int PeriodK
		{
			get { return periodK; }
			set { periodK = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for smoothing the slow K values")]
		[Category("Parameters")]
		public int Smooth
		{
			get { return smooth; }
			set { smooth = Math.Max(1, value); }
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TheForceStochastics[] cacheTheForceStochastics = null;

        private static TheForceStochastics checkTheForceStochastics = new TheForceStochastics();

        /// <summary>
        /// The TheForceStochastics Oscillator is made up of two lines that oscillate between a vertical scale of 0 to 100. The %K is the main line and it is drawn as a solid line. The second is the %D line and is a moving average of %K. The %D line is drawn as a dotted line. Use as a buy/sell signal generator, buying when fast moves above slow and selling when fast moves below slow.
        /// </summary>
        /// <returns></returns>
        public TheForceStochastics TheForceStochastics(int periodD, int periodK, int smooth)
        {
            return TheForceStochastics(Input, periodD, periodK, smooth);
        }

        /// <summary>
        /// The TheForceStochastics Oscillator is made up of two lines that oscillate between a vertical scale of 0 to 100. The %K is the main line and it is drawn as a solid line. The second is the %D line and is a moving average of %K. The %D line is drawn as a dotted line. Use as a buy/sell signal generator, buying when fast moves above slow and selling when fast moves below slow.
        /// </summary>
        /// <returns></returns>
        public TheForceStochastics TheForceStochastics(Data.IDataSeries input, int periodD, int periodK, int smooth)
        {
            checkTheForceStochastics.PeriodD = periodD;
            periodD = checkTheForceStochastics.PeriodD;
            checkTheForceStochastics.PeriodK = periodK;
            periodK = checkTheForceStochastics.PeriodK;
            checkTheForceStochastics.Smooth = smooth;
            smooth = checkTheForceStochastics.Smooth;

            if (cacheTheForceStochastics != null)
                for (int idx = 0; idx < cacheTheForceStochastics.Length; idx++)
                    if (cacheTheForceStochastics[idx].PeriodD == periodD && cacheTheForceStochastics[idx].PeriodK == periodK && cacheTheForceStochastics[idx].Smooth == smooth && cacheTheForceStochastics[idx].EqualsInput(input))
                        return cacheTheForceStochastics[idx];

            TheForceStochastics indicator = new TheForceStochastics();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.PeriodD = periodD;
            indicator.PeriodK = periodK;
            indicator.Smooth = smooth;
            indicator.SetUp();

            TheForceStochastics[] tmp = new TheForceStochastics[cacheTheForceStochastics == null ? 1 : cacheTheForceStochastics.Length + 1];
            if (cacheTheForceStochastics != null)
                cacheTheForceStochastics.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTheForceStochastics = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The TheForceStochastics Oscillator is made up of two lines that oscillate between a vertical scale of 0 to 100. The %K is the main line and it is drawn as a solid line. The second is the %D line and is a moving average of %K. The %D line is drawn as a dotted line. Use as a buy/sell signal generator, buying when fast moves above slow and selling when fast moves below slow.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TheForceStochastics TheForceStochastics(int periodD, int periodK, int smooth)
        {
            return _indicator.TheForceStochastics(Input, periodD, periodK, smooth);
        }

        /// <summary>
        /// The TheForceStochastics Oscillator is made up of two lines that oscillate between a vertical scale of 0 to 100. The %K is the main line and it is drawn as a solid line. The second is the %D line and is a moving average of %K. The %D line is drawn as a dotted line. Use as a buy/sell signal generator, buying when fast moves above slow and selling when fast moves below slow.
        /// </summary>
        /// <returns></returns>
        public Indicator.TheForceStochastics TheForceStochastics(Data.IDataSeries input, int periodD, int periodK, int smooth)
        {
            return _indicator.TheForceStochastics(input, periodD, periodK, smooth);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The TheForceStochastics Oscillator is made up of two lines that oscillate between a vertical scale of 0 to 100. The %K is the main line and it is drawn as a solid line. The second is the %D line and is a moving average of %K. The %D line is drawn as a dotted line. Use as a buy/sell signal generator, buying when fast moves above slow and selling when fast moves below slow.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TheForceStochastics TheForceStochastics(int periodD, int periodK, int smooth)
        {
            return _indicator.TheForceStochastics(Input, periodD, periodK, smooth);
        }

        /// <summary>
        /// The TheForceStochastics Oscillator is made up of two lines that oscillate between a vertical scale of 0 to 100. The %K is the main line and it is drawn as a solid line. The second is the %D line and is a moving average of %K. The %D line is drawn as a dotted line. Use as a buy/sell signal generator, buying when fast moves above slow and selling when fast moves below slow.
        /// </summary>
        /// <returns></returns>
        public Indicator.TheForceStochastics TheForceStochastics(Data.IDataSeries input, int periodD, int periodK, int smooth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TheForceStochastics(input, periodD, periodK, smooth);
        }

    }
}
#endregion
